@@ -3,7 +3,7 @@ package Leyland::Controller;
 # ABSTRACT: Leyland controller base class
 
 use Moose::Role;
-use MooseX::ClassAttribute;
+use MooseX::ClassAttribute 0.24;
 use namespace::autoclean;
 
 =head1 NAME
@@ -12,7 +12,7 @@ Leyland::Controller - Leyland controller base class
 
 =head1 VERSION
 
-version 0.001005
+version 0.001006
 
 =head1 SYNOPSIS
 
